import os

from github.Consts import DEFAULT_BASE_URL as GITHUB_DEFAULT_BASE_URL


class GitHubEnvironment:
    def get_token(self) -> str:
        """Gets the authentication token for this environment."""
        token = os.environ.get("GITHUB_ENTERPRISE_TOKEN", os.environ.get("GITHUB_TOKEN"))
        if not token:
            raise ValueError("GITHUB_ENTERPRISE_TOKEN or GITHUB_TOKEN must be set")

        return token

    def get_url(self) -> str:
        """Gets the URL for the git provider."""
        return os.environ.get("GH_HOST", GITHUB_DEFAULT_BASE_URL)



import logging
import os
import re
from datetime import datetime
from typing import Literal, Optional

import duckdb
import pandas as pd
from rich.spinner import Spinner

from prfiesta.spinner import update_spinner

logger = logging.getLogger(__name__)

OUTPUT_TYPE = Literal["csv", "parquet", "duckdb"]
WIN_ILLEGAL_FILENAME = r"[:\/\\\*\?\"\<\>\|]"


def output_frame(
    frame: pd.DataFrame,
    output_type: OUTPUT_TYPE,
    spinner: Spinner,
    output_name: Optional[str] = None,
    timestamp: Optional[datetime] = None,
) -> None:
    if not timestamp:
        timestamp = datetime.now()

    if not output_name:
        output_name = f"export.{timestamp.strftime('%Y%m%d%H%M%S')}.{output_type}"

    if os.name == "nt" and re.search(WIN_ILLEGAL_FILENAME, output_name):
        msg = f"{output_name} is an invalid filename on windows"
        raise ValueError(msg)

    update_spinner(f"Writing export to {output_name}", spinner, logger)

    if output_type == "csv":
        frame.to_csv(output_name, index=False)

    elif output_type == "parquet":
        frame.to_parquet(output_name, index=False)

    elif output_type == "duckdb":
        duckdb_table = f"prfiesta_{timestamp.strftime('%Y%m%d_%H%M%S')}"

        update_spinner(f"connecting to duckdb {output_name}", spinner, logger)
        conn = duckdb.connect(output_name)

        update_spinner(f"exporting to duckdb table {duckdb_table}", spinner, logger)
        conn.execute(f"CREATE TABLE {duckdb_table} AS SELECT * FROM frame")  # noqa: S608 duckdb_table is always constructed internally

        conn.close()
    else:
        raise ValueError("unknown output_type %s", output_type)

    logger.info("Exported to %s!", output_name)



from datetime import datetime, timezone

import pandas as pd
from IPython.display import HTML, DisplayObject
from natural.date import duration


def _enrich_pr_link(data: pd.DataFrame) -> pd.DataFrame:
    def make_link(row: pd.Series) -> str:
        return f'<a href="{row["html_url"]}">{row["title"]}</a>'

    data["title"] = data.apply(make_link, axis=1)
    return data.drop(columns="html_url")


def view_pull_requests(data: pd.DataFrame, **kwargs) -> DisplayObject:
    as_frame: bool = kwargs.get("as_frame", False)
    relative_dates: bool = kwargs.get("relative_dates", True)
    head: int = kwargs.get("head")

    temp = data[["number", "title", "repository_name", "updated_at", "html_url"]].copy()
    temp = _enrich_pr_link(temp)

    if relative_dates:
        temp["updated_at"] = pd.to_datetime(temp["updated_at"])
        temp["updated_at"] = temp["updated_at"].apply(duration, now=datetime.now(timezone.utc))

    if head:
        temp = temp.head(head)

    if as_frame:
        return temp

    return HTML(temp.to_html(escape=False, index=False))
